package org.openl.rules.testmethod.result;

import java.lang.reflect.Method;
import java.util.List;

import org.openl.exception.OpenlNotCheckedException;
import org.openl.rules.testmethod.TestUnitResultComparator.TestStatus;
import org.openl.rules.testmethod.result.ComparedResult;
import org.openl.util.StringTool;

public class BeanResultComparator implements TestResultComparator {
    private List<String> fieldsToCompare;
    private List<ComparedResult> comparisonResults;
    
    public BeanResultComparator(List<String> fieldsToCompare) {
        if (fieldsToCompare != null) {
            this.fieldsToCompare = fieldsToCompare;
        } else {
            throw new IllegalArgumentException("Fields for comparing cannot be null");
        }
    }
    
    public List<String> getFieldsToCompare() {
        return fieldsToCompare;
    }

    public List<ComparedResult> getComparisonResults(){
        return comparisonResults;
    }
    
    public boolean compareResult(Object actualResult, Object expectedResult) {
        if (actualResult == null || expectedResult == null) {
        	return actualResult == expectedResult;
        } else {
        	boolean success = true;
        	for (String fieldToCompare : fieldsToCompare) {
        	    ComparedResult fieldComparisonResults = new ComparedResult();
        	    fieldComparisonResults.setFieldName(fieldToCompare);

        	    Object actualFieldValue = getFieldValue(actualResult, fieldToCompare);
                Object expectedFieldValue = getFieldValue(expectedResult, fieldToCompare);

                fieldComparisonResults.setActualValue(actualFieldValue);
                fieldComparisonResults.setExpectedValue(expectedFieldValue);
                
                TestResultComparator comparator = TestResultComparatorFactory.getComparator(actualFieldValue, expectedFieldValue);
                boolean compare = comparator.compareResult(actualFieldValue, expectedFieldValue);
                
                
                if (!compare) {
                    fieldComparisonResults.setStatus(TestStatus.TR_NEQ);
                    success = false;
                } else {
                    fieldComparisonResults.setStatus(TestStatus.TR_OK);
                }
            }
            return success;
        }
        
    }
    
    protected Object getFieldValue(Object target, String fieldName) {
        Object res = null;
        Class<?> targetClass = target.getClass();
        Method method;
        try {
            method = targetClass.getMethod(StringTool.getGetterName(fieldName), new Class<?>[0]);
            res = method.invoke(target, new Object[0]);
        } catch (Exception e1) {
            String message = String.format("Cannot get value for field %s", fieldName);
            throw new OpenlNotCheckedException(message, e1);
        }
        return res;
    }

}
